#----------------------------------------------------------------
#
# Mapmaker version 2.51 (Python version)  Last revised 08/26/05
#
# By John Trauger (some elements from version 1 witten by Tim Sargent)
#
# vorlon@vorlonagent.com
#
#----------------------------------------------------------------

import sys
import operator
import xlbook
import copy

def diagVar(functName, varName, varValue):

    sys.stdout.write(functName)
    sys.stdout.write(" : ")
    sys.stdout.write(varName)
    sys.stdout.write(" >")
    sys.stdout.write(varValue)
    sys.stdout.write("< ")

# End of diagVar


class GameStuff:

# Keeps track of Game-wide variables and misc stuff that might otherwise
# be assigned to a global variable.  Also includes important reverse-
# lookup tables.  This structure is created once.
    
    currTurn = "0"
    currImpulse = "0"
    nSides = 2
    gameName = "DEFAULT"
    counterPath = "DEFAULT"
    ssdPath = "DEFAUKT"
    mapPath = "DEFAULT"
    sitrepPath = ""
    sitrepFileName = "DEFAULT"
    validSides = {}
    validDirections = "ABCDEF"
    validSideClasses = "S,B,T,M,P,PF,I,BO,G" 
    validMinorClasses = "S,F,D,P,PF,M,T,I,BO,W"  
    validNeutralClasses = "SMAPT"
    stackThreshold = 6
    unitArrange = ["n/a",[],[],[],[],[],[],[],[]]
    unitOffset = [[],[],[],[],[],[],[],[],[]]
    facingOffset = {"a":0,"b":1,"c":3,"d":4,"e":5,"f":7}
    positionList = {}
    classLookup = {}
    backgroundClasses = []

# Index Box colors
    titleTextColor = "#000000"  # Black
    titleColor = "#C0C0C0"      # Gray
    bodyTextColor = "#000000"   # White
    bodyColor = "#FFFFFF"       # Black


    def __init__(self):
        self.unitArrange[1] = [0]
        self.unitArrange[2] = [5,1]
        self.unitArrange[3] = [7,5,2]
        self.unitArrange[4] = [7,5,3,1]
        self.unitArrange[5] = [7,6,5,3,1]
        self.unitArrange[6] = [7,6,5,8,4,2]
        self.unitArrange[7] = [7,6,5,8,4,1,3]
        self.unitArrange[8] = [7,6,5,8,4,1,2,3]

        self.stackThreshold = 6

        self.unitOffset[0] = "0,0"
        self.unitOffset[1] = "-1,-1"
        self.unitOffset[2] = "-1,0"
        self.unitOffset[3] = "-1,1"
        self.unitOffset[4] = "0,1"
        self.unitOffset[5] = "1,1"
        self.unitOffset[6] = "1,0"
        self.unitOffset[7] = "1,-1"
        self.unitOffset[8] = "0,-1"


class SideStuff:
# This structure is intantiated once for each distinct side in a
# scenario plus one extra for "neutral" phenominon.
    
    sideName = "DEFAULT"
    counterFolder = "default/"
    desigLetter = "DF"
    unitList = {}
    counterList = {}

# Info Boxes
    titleTextColor = "DEFAULT"
    titleColor = "DEFAULT"
    bodyTextColor = "DEFAULT"
    bodyColor = "DEFAULT"


class HexStuff:
# This structure holds information for each hex on the map that is
# occupied by one or more units: A list of the units in the hex
# (.unitList), the tooltip display information (.tooltips), and the
# info box display html (.infoBoxes). Each line of info box data gets
# its own entry in infoBoxes[].
    
    unitList = []
    backgrList = []
    tooltips = ""
    infoBoxes = []

    
class DataFields:
# Contains information telling the system how to parse a sitrep
# Each type of unit in DataStuff below needs the same set of
# information tracked for it: The list of designator(s) for that
# class (.desig), What fields will be read for that class (.read),
# What field(s) will go in the info box header (.boxTitle), what
# fields will go in the body of the info box (.boxData) and what
# fields will be placed in the tooltip for the hex(.tooltip)
##    desigList = "z"
    boxTitle = "Unit"
    boxData = "Hex,Speed,EW"
    tooltip = "unitID,Hex,Speed,EW"


class UnitStuff:
# Stores all data for an individual unit.  Since properties
# can be added after the fact, not all possible properties
# are shown from the point of definition.

    unitID = ""
    ssdFn = ""
    counterFn = "arrow"
    hex = "0000"
    displayOffsetLeft = 0
    displayOffsetTop = 0
    facing = "Z"
    speed = "-40"
    notes = ""
    infoBox = []
    owningSide = 0
    unitClass = "ca"
    ecm = "-10"
    eccm = "-10"
    useNeutral = False
    oddCounterSize = False
    topOffset = 4
    leftOffset = 4
    formatCode = "a:"
    startHex = "0000"
    endHex = "0000"
    radiusHexs = 5
    pixelWidth = "20"
    pixelHeight = "20"
    controllingUnitID = "NU1"
    isTractoring = False
    tractoredList = []
    isTractored = False
    tracotringUnitID = ""
    tooltipStuff = ""

# End of UnitStuff

class MapStuff:
    mType = "0"
    mBackFN = "blackPlainMap.gif"
    mHeight = "1565"
    mWidth = "1277"
    mOutPath = ""
    mOutFileFormat = "blackMap.htm"
    mOutFilename = "blackMap.htm"
    mToolTips = False
    mInfoBoxes = False
    mLeadingZeroes = False
    numMapFiles = (0, 1)

    leftOffset = 0
    topOffset = 0
    evenTopOffset = 0
    
    leftOneHex = 29
    downOneHex = 34

    adTopOffset = 4
    adLeftOffset = 4
    bcTopOffset = -4
    bcLeftOffset = -4
    efTopOffset = -4
    efLeftOffset = -5

    clusterOffset = 7
    stackTopOffset = -2
    stackLeftOffset = 2
    stackTopMod = 10
    stackLeftMod = -10


    
    
    def __init__(self, mapType):
   
#  42x42 hex un-numbered FOG2 map.
        if (mapType == "0"):
            self.mHeight = "1565"
            self.mWidth = "1277"

            self.leftOffset = 1
            self.topOffset = -14
            self.evenTopOffset = 17

#  42x42 hex numbered FOG2 map.
        elif (mapType == "1"):
            self.mHeight = "1722"
            self.mWidth = "1466"

            self.leftOffset = -22
            self.topOffset = -30
            self.evenTopOffset = 17


        else:
            sys.stdout.write("SETNEWMAP ERROR : Invalid Map Type >")
            sys.stdout.write(mapType)
            sys.stdout.write("<")
            # error reporting goes here

    def MapfileName(self, mapFormat, gameStuff):
        self.OutFileFormat = mapFormat

        # Replace <T>, <I> with turn and impulse
        mapFormat = mapFormat.replace("<t>",gameStuff.currTurn)
        mapFormat = mapFormat.replace("<T>",gameStuff.currTurn)
        mapFormat = mapFormat.replace("<i>",gameStuff.currImpulse)
        mapFormat = mapFormat.replace("<I>",gameStuff.currImpulse)
            
        self.mOutFilename = mapFormat + ".htm"
        
# End of mapStuff



def makeLicensePlate(vName):

    vName
    lPlate = ""
    
    vNameSplit = vName.split(".")
    for i in vNameSplit:
        lPlate += i

    return lPlate

# End of makeLicensePlate



def iniRead(iniPath,iniFileName):
    
    ShowComments = False

    iniFile = open(iniPath + iniFileName,"r")
    inFile = iniFile.readlines()

    iniVars = []

    for inStuff in inFile:
        isError = False
        evalThis = True
        inStuff=inStuff.strip()
        inStuffSplit = inStuff.split("=")
        if (ShowComments):
            sys.stdout.write("Ini line: >" + inStuff + "<\n")
        
        if (inStuff == ""):
            outStuff = "No input"
            evalThis = False

        elif (inStuff[0] == "#"):
            outStuff = "Comment"
            evalThis = False
                 
        elif (len(inStuffSplit) < 2):
            outStuff = "IniRead - Bad Expression: >" + inStuff + "<"
            evalThis = False
            isError = True

        else:
            inVar = inStuffSplit[0].strip()
            inVal = inStuffSplit[1].strip()
            if (ShowComments):
                outStuff = "inVar = >" + inVar + "<\n" + "inVal = >" + inVal + "<\n"
                sys.stdout.write(outStuff)

            inValLow = inVal.lower()

            isNumeric = True
            for i in inVal:
                if (i not in "1234567890."):
                    isNumeric = False

            if (isNumeric):
                pass

            elif ((inValLow == "true") or (inValLow == "yes")):
                inVal = "True"
        
            elif ((inValLow == "false") or (inValLow == "no")):
                inVal = "False"
        
            elif ((inVal[0] == "'") and (inVal[len(inVal)-1] == '"')):
                inVal = inVal.replace("'",'"',1)

            elif ((inVal[0] == '"') and (inVal[len(inVal)-1] == "'")):
                inVal = inVal.replace('"',"'",1)

        if (evalThis):
            outStuff = [inVar, inVal]
            iniVars += [outStuff]

        if ((ShowComments) or (isError)):
            sys.stdout.write(outStuff)
            sys.stdout.write("\n\n")

    if (ShowComments):
        sys.stdout.write("iniVars = ")
        sys.stdout.write(iniVars)
        sys.stdout.write("\n" + "\n")

    return iniVars

# End of iniRead


def buildBoxLines(fieldSplit,unitInfo,widthStuff,sideLetter,lineType,gameStuff):

    outLines = ""
    wLength = len(widthStuff)
    
    for i in range(len(fieldSplit)):
        cField = fieldSplit[i]
        cFieldSplit = cField.split("/")
        cField = cFieldSplit[0].lower()
        lineFormat = ""
        if (len(cFieldSplit) > 1):
            lineFormat = cFieldSplit[1:]
        
        outLines += '<TR>\n'
        skipField = False
        if (cField == "unit"):
            dData = [unitInfo.unitID]
            if(wLength == 2):
                dData = ["Unit:"] + dData

        elif (cField == "type"):
            dData = [unitInfo.unitClass]
            if(wLength == 2):
                dData = ["Class:"] + dData

        elif (cField == "name"):
            dData = [unitInfo.name]
            if(wLength == 2):
                dData = ["Name:"] + dData
                
        elif (cField == "hex"):
            dData = [unitInfo.hex]
            if(wLength == 2):
                dData = ["Hex:"] + dData
                
        elif (cField == "speed"):
            dData = [unitInfo.speed]
            if(wLength == 2):
                dData = ["Speed:"] + dData
                
        elif (cField == "ew"):
            dData = [unitInfo.ecm + "/" + unitInfo.eccm]
            if(wLength == 2):
                dData = ["EW:"] + dData
                
        elif (cField == "launch"):
            dData = [unitInfo.launch]
            if(wLength == 2):
                dData = ["Launched:"] + dData

        elif (cField == "laid"):
            dData = [unitInfo.launch]
            if(wLength == 2):
                dData = ["Laid:"] + dData

        elif (cField == "cast"):
            dData = [unitInfo.launch]
            if(wLength == 2):
                dData = ["Cast:"] + dData

        elif (cField == "publicnotes"):
            dData = [unitInfo.notes]
            if(dData == [""]):
                skipField = True
            else:
                if(wLength == 2):
                    wLength = 1
                    unitNotes = "Notes:<br>"
                else:
                    unitNotes = ""

                noteStuff = dData[0].split(",")
                for j in noteStuff:
                    unitNotes += "&nbsp&nbsp "+ j + "<br>"

                dData=[unitNotes]

        else:
            skipField = True

        if (not(skipField)):
            for j in range(wLength):
                if (j == 0):
                    outLines += '   <TD class=' + sideLetter.lower() + lineType.title() + 'Left'
                    if ((wLength == 1) and (i > 0)):
                        outLines += ' colspan="2" '
                else:
                    outLines += '   <TD class=' + sideLetter.lower() + lineType.title() + 'Right'

                outLines += ' noWrap >'
                outLines += dData[j]
                
                if ((lineType == "Title") and (i == 0)):
                    outLines += '</TD>\n   <TD class=' + sideLetter.lower() + lineType.title() + 'Right noWrap >'
                    outLines += '<IMG src="' + gameStuff.mapPath + 'exit_x.gif" border=1 width="12" height="12"'

                    currHex = unitInfo.hex
                    hexData = gameStuff.positionList[currHex]
                    unitList = hexData.unitList
                    if (len(unitList) == 1):
                        outLines += ' onclick="JavaScript:closeParent(' + "'hex" + currHex + "'" + ');">\n'
                    else:
                        unitID = unitInfo.unitID
                        layerID = "" + makeLicensePlate(unitID)
                        outLines += ' onclick="JavaScript:closeChild(' + "'" + layerID + "'" + ');">\n'

                outLines += '</TD>\n'
        outLines += '</TR>\n'

    return outLines


# End of buildBoxLines


def buildUnitBox(whichSide,unitID,unitInfo,dataRef,mapInfo,gameStuff):

    ShowComments = False

    tempBoxStuff = ""
    titleSplit = dataRef.boxTitle.split(",")
    tempBoxStuff += buildBoxLines(titleSplit,unitInfo,["100"],whichSide.desigLetter,"Title",gameStuff)

    if (ShowComments):
        sys.stdout.write("BUILDUNITBOX: tempBoxStuff = >" + tempBoxStuff + "<\n")
    
    dataSplit = dataRef.boxData.split(",")
    tempBoxStuff += buildBoxLines(dataSplit,unitInfo,["40","60"],whichSide.desigLetter,"Body",gameStuff)

    if (unitInfo.ssdFn):
        ssdSplit = unitInfo.ssdFn.split(".")
        ssdFn = ssdSplit[0]
        ssdPath = gameStuff.ssdPath + ssdFn.lower()
        tempBoxStuff += '<TR>\n'
        tempBoxStuff += '   <TD class=' + whichSide.desigLetter.lower() + 'BodyLeft noWrap colSpan=2>'
        tempBoxStuff += '<INPUT class=ssdButton onClick = "displaySSD(' + "'" + ssdPath  + "'" + ')"  type=button value=SSD></TD>\n'
        tempBoxStuff += '</TR>\n'
        if (ShowComments):
            sys.stdout.write("BUILDUNITBOX : ssdFn = " + ssdFn + " ssdPath = " + ssdPath + "\n")

    unitInfo.infoBox = tempBoxStuff
    return tempBoxStuff


# End of buildUnitBox


def buildChildBox(unitID,hexData,mapInfo,gameStuff):

    ShowComments = False

    layerID = makeLicensePlate(unitID)
    childBoxStuff =  '<DIV ID="'+layerID+'" STYLE="POSITION: absolute; LEFT: 150px; TOP: 0px; VISIBILITY: hidden; Z-INDEX: 4;">\n'
    childBoxStuff += '<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0">\n<TBODY>\n'    # WIDTH = "74"

    sDesig = unitID[0]
    foundSide = gameStuff.validSides.has_key(sDesig)
    if (foundSide) :
        sideInfo = gameStuff.validSides[sDesig]
        unitInfo = sideInfo.unitList[unitID]
        unitIDSplit = unitID.split(".")
        uClass = getUnitClass(unitIDSplit)
        dataRef = gameStuff.classLookup[uClass]
        childBoxStuff += buildUnitBox(sideInfo,unitID,unitInfo,dataRef,mapInfo,gameStuff)

    childBoxStuff += '</TBODY>\n</TABLE></DIV>\n'

    if (ShowComments):
        sys.stdout.write("\nBUILDCHILDBOX : childBoxStuff = " + childBoxStuff + "\n")
                   
    return childBoxStuff

                    
# End of buildChildBox



def buildParentBox(currHex,hexData,mapInfo,gameStuff):

    ShowComments = False

    unitList = hexData.unitList
    unitList.sort()
    uLength = len(unitList)

    parentBoxStuff =  '<DIV ID="hex' + currHex + '" STYLE="POSITION: absolute; LEFT: 0px; TOP: 0px; VISIBILITY: hidden; Z-INDEX: 4;">\n'
    parentBoxStuff += '<TABLE BORDER="0" CELLSPACING="0" CELLPADDING="0" width = "74">\n'

    if (uLength ==1):
        unitID = unitList[0]
        sDesig = unitID[0]
        foundSide = gameStuff.validSides.has_key(sDesig)
        if (foundSide) :
            sideInfo = gameStuff.validSides[sDesig]
            unitInfo = sideInfo.unitList[unitID]
            unitIDSplit = unitID.split(".")
            uClass = getUnitClass(unitIDSplit)
            dataRef = gameStuff.classLookup[uClass]
            parentBoxStuff += buildUnitBox(sideInfo,unitID,unitInfo,dataRef,mapInfo,gameStuff)
        else:
            pass #Error Condition

    else:
        parentBoxStuff += '<TBODY>\n   <TR>\n      <TD class=parentTitleLeft'
        parentBoxStuff += ' noWrap >Hex ' + currHex
        parentBoxStuff += '</TD>\n      <TD class=parentTitleRight noWrap >'
        parentBoxStuff += '<IMG src="' + gameStuff.mapPath + 'exit_x.gif" border=0 width="12" height="12"'
        parentBoxStuff += ' onclick="JavaScript:closeParent(' + "'hex" + currHex + "'" + ');">'
        parentBoxStuff += '      </TD>\n   </TR>\n'

        for m in unitList:
            uIdent = makeLicensePlate(m)
            parentBoxStuff += '   <TR><TD class=parentBody'
            parentBoxStuff += ' noWrap colspan=2><A class=parentLink href="javascript:showChild(' + "'"
            parentBoxStuff += uIdent + "'" + ');">' + m + '</A></TD></TR>\n'
        
    parentBoxStuff += '</TBODY>\n</TABLE></DIV>\n'

    if (ShowComments):
        sys.stdout.write("BUILDPARENTBOX : parentBoxStuff = " + parentBoxStuff + "\n")

    return parentBoxStuff

                    
# End of buildParentBox


def formatOddCounter(mapInfo, unitInfo):

    formatCode = unitInfo.formatCode.lower()
    formatSplit = formatCode.split(":")
    formatType = formatSplit[0].strip()
    placement = formatSplit[1].strip()
    cornerTopOffset = -2
    cornerLeftOffset = 0
    hexTopOffset = -3
    hexLeftOffset =  mapInfo.downOneHex - mapInfo.leftOneHex-10

    # determine size of the counter
    if (formatType == "a"): # Absolute counter size
        # Width and height properties should be set when the counter is added in readSitrep

        # Begin placmeent
        if ("hc" in placement): # center counter in hex
            unitInfo.topOffset = (mapInfo.leftOneHex - int(unitInfo.pixelHeight)) / 2
            unitInfo.leftOffsetY = (mapInfo.downOneHex - int(unitInfo.pixelWidth)) / 2
            
        elif ("c" in placement): # position in upper-left of hex
            unitInfo.topOffset = cornerTopOffset
            unitInfo.leftOffset = cornerLeftOffset

        elif ("h" in placement): # position in upper-left of box at hex height a wide as hex a widest point
            unitInfo.topOffset = hexTopOffset
            unitInfo.leftOffset = hexLeftOffset
        
    
    elif (formatType == "h"): # counter is dimensions of a hex
        unitInfo.pixelWidth = str((mapInfo.leftOneHex - 2) *4 /3)
        unitInfo.pixelHeight = str(mapInfo.downOneHex)
        unitInfo.topOffset = hexTopOffset
        unitInfo.leftOffset = hexLeftOffset

    elif (formatType == "c"): # counter is circle with a set radius
        unitInfo.pixelWidth = str(int(mapInfo.leftOneHex) * ((2 * unitInfo.radiusHexs) + 1))
        unitInfo.pixelHeight = str(int(mapInfo.downOneHex) * ((2 * unitInfo.radiusHexs) + 1))
        unitInfo.topOffset = hexTopOffset - str(int(mapInfo.leftOneHex) * unitInfo.radiusHexs)
        unitInfo.leftOffset = hexLeftOffset - str(int(mapInfo.leftOneHex) * unitInfo.radiusHexs)

    elif (formatType == "r"): # counter is rectangle with set endpoints, either hexes or ship positions
        detailSplit = placement.split("-")
        rectKind = detailSplit[0]
        # format for hex size: h is set hex numbers, u = get a unit's hex
        # need function to take a unitID and come back with a hex

        rectEdges = detailSplit[1]
        # format same as absolute.  hc builds from/to center of hex, c goes to corner h designed to fill the hex


        


# end of formatoddCounter

def buildMapHex(currHex,hexData,mapInfo,gameStuff):

    ShowComments = False

    hexCols = int(currHex[0:2])
    hexRows = int(currHex[2:4])
    coarseLeftPos = (hexCols * mapInfo.leftOneHex) + mapInfo.leftOffset
    coarseTopPos = (hexRows * mapInfo.downOneHex) + mapInfo.topOffset

    unitList = hexData.unitList
    nUnits = len(unitList)
    if (nUnits < gameStuff.stackThreshold):
        spanWidth = 43
        spanHeight = 43
    else:
        spanWidth = 29 + (nUnits * 2)
        spanHeight = 29 + (nUnits * 2)
        stackTopMod = mapInfo.stackTopMod - nUnits/2
        if (stackTopMod < -10):
            stackTopMod = -10
        
        stackLeftMod = mapInfo.stackLeftMod + nUnits/2
        if (stackLeftMod > 10):
            stackLeftMod = 10


    if (hexCols % 2 == 0):
        coarseTopPos += mapInfo.evenTopOffset

    hTooltips = ""

    outLines = '<SPAN id=spn' + currHex + ' style="Z-INDEX: 2; LEFT: ' + str(coarseLeftPos) + 'px;'
    outLines += ' WIDTH: ' + str(spanWidth) + 'px; POSITION: absolute; TOP: ' + str(coarseTopPos) + 'px;'
    outLines += ' HEIGHT: ' + str(spanHeight) + 'px">\n'

    if (nUnits > 1):
        unitList.sort()
        unitList.reverse()
    
    primeUnitFacing = ""


    if (ShowComments):
        sys.stdout.write("BUILDMAPHEX : nUnits = " + str(nUnits) + "\n")

    for k in range(nUnits):
        sDesig = unitList[k][0]
        foundSide = gameStuff.validSides.has_key(sDesig)

        if (ShowComments):
            sys.stdout.write("BUILDMAPHEX : k = >" + str(k) + "< sDesig =>" + str(sDesig) + "< foundSide =>" + str(foundSide)+ "< gameStuff.validSides.keys() =>" + str(gameStuff.validSides.keys())  + "\n")


        if (foundSide) :
            if (ShowComments):
                sys.stdout.write("\nBUILDMAPHEX : nUnits = ")
                sys.stdout.write(nUnits)
                sys.stdout.write(" k = ")
                sys.stdout.write(k)
                sys.stdout.write(" unitList[k] = >" + unitList[k])
                sys.stdout.write("< currHex = >" + currHex)
                sys.stdout.write("< coarseLeftPos = >")
                sys.stdout.write(coarseLeftPos)
                sys.stdout.write("< coarseTopPos = >")
                sys.stdout.write(coarseTopPos)
                sys.stdout.write("<\n")

            sideInfo = gameStuff.validSides[sDesig]
            unitInfo = sideInfo.unitList[unitList[k]]

            sDesigLower = sDesig.lower()
            unitFacing = unitInfo.facing
            if (unitFacing not in gameStuff.validDirections):
                unitFacing = "a"
                    
            unitFacing = unitFacing.lower()
            if (unitInfo.oddCounterSize == True):
                formatOddCounter(mapInfo, unitInfo)
                unitLeftOffset = unitInfo.leftOffset
                unitTopOffset = unitInfo.topOffset
            elif ((unitFacing == "a") or (unitFacing == "d")):
                unitLeftOffset = mapInfo.adLeftOffset
                unitTopOffset = mapInfo.adTopOffset
                unitInfo.pixelWidth = "20"
                unitInfo.pixelHeight = "20"
            else :
                unitLeftOffset = 0
                unitTopOffset = 0
                unitInfo.pixelWidth = "29"
                unitInfo.pixelHeight = "29"

            if (nUnits == 1):
                unitArrange = gameStuff.unitOffset[0]
                unitArrangeLeft = int(unitArrange.split(",")[1]) * mapInfo.clusterOffset
                unitArrangeTop = int(unitArrange.split(",")[0]) * mapInfo.clusterOffset
            elif (nUnits >= gameStuff.stackThreshold):
                unitArrangeLeft = (mapInfo.stackLeftOffset * k) + stackLeftMod
                unitArrangeTop = (mapInfo.stackTopOffset * k) + stackTopMod
            else:
                hexArrange = gameStuff.unitArrange[nUnits]
                if (primeUnitFacing == ""):
                    primeUnitFacing = unitFacing
                    primeUnitOffset = gameStuff.facingOffset[primeUnitFacing]

                    if (ShowComments):
                        sys.stdout.write("BUILDMAPHEX : primeUnitFacing = >" + primeUnitFacing)
                        sys.stdout.write("< primeUnitOffset = >")
                        sys.stdout.write(primeUnitOffset)
                        sys.stdout.write("<\n")


                unitArrange = (hexArrange[k] + primeUnitOffset) % 8
                if (unitArrange == 0):
                    unitArrange = 8
                unitArrange = gameStuff.unitOffset[unitArrange]

                if (ShowComments):
                    sys.stdout.write("BUILDMAPHEX : unitArrange = " + unitArrange + "\n")

                unitArrangeLeft = int(unitArrange.split(",")[1]) * mapInfo.clusterOffset
                unitArrangeTop = int(unitArrange.split(",")[0]) * mapInfo.clusterOffset


            if (ShowComments):
                sys.stdout.write("BUILDMAPHEX : unitLeftOffset = >" + str(unitLeftOffset) + "< unitArrangeLeft = >" + str(unitArrangeLeft) + "<\n")
                sys.stdout.write("BUILDMAPHEX : unitTopOffset = >" + str(unitLeftOffset) + "< unitTopOffset = >" + str(unitTopOffset) + "<\n")
                        
            finalUnitLeft = unitLeftOffset + unitArrangeLeft
            finalUnitTop = unitTopOffset + unitArrangeTop

            if (ShowComments):
                sys.stdout.write("BUILDMAPHEX : unitArrangeLeft = >")
                sys.stdout.write(unitArrangeLeft)
                sys.stdout.write("< unitArrangeTop = >")
                sys.stdout.write(unitArrangeTop)
                sys.stdout.write("< finalUnitLeft = >")
                sys.stdout.write(finalUnitLeft)
                sys.stdout.write("< finalUnitTop = >")
                sys.stdout.write(finalUnitTop)
                sys.stdout.write("<\n")


            unitInfo.displayOffsetLeft = finalUnitLeft
            unitInfo.displayOffsetTop = finalUnitTop

            counterFN = "/" + unitInfo.counterFn
            uniqueID = makeLicensePlate(unitInfo.unitID)

            counterFolder = sideInfo.counterFolder
            if (unitInfo.useNeutral):
                counterFolder = "neutral"
                
            outLines += '<IMG src="' + gameStuff.counterPath + counterFolder + counterFN + '-'+ unitInfo.facing.lower() + '.gif"'
            outLines += ' id=img' + uniqueID

            if (mapInfo.mToolTips):
                hexStuff = ' alt="' + hexData.tooltips + '" '
                hTooltips += hexData.tooltips
            else:
                hexStuff = ''

            if (mapInfo.mInfoBoxes):
                if (nUnits == 1):
                    infoStuff = 'onClick="JavaScript:showParentChild(' + "'" + 'hex' + currHex + "'" +');"'
                else:
                    infoStuff = 'onClick="JavaScript:showParentChild(' + "'" + 'hex' + currHex
                    infoStuff += "','" + uniqueID + "'" +');"'
            else:
                infoStuff = ''

            outLines += hexStuff + '\n'
            outLines += ' style="Z-INDEX: 2; LEFT: ' + str(finalUnitLeft) + 'px;'
            outLines += ' WIDTH: ' + unitInfo.pixelWidth + 'px;'
            outLines += ' POSITION: absolute; TOP: ' + str(finalUnitTop) + 'px;'
            outLines += ' HEIGHT: ' + unitInfo.pixelHeight + 'px"\n'
            outLines += infoStuff + '>\n'
                
    outLines += '</SPAN>\n'

    return outLines
    
# End of buildMapHex


def buildMapFile(mapInfo, gameStuff, sideStuff, mapStuff, dataStuff):

    ShowComments = False

    if (ShowComments):
        sys.stdout.write("MAPSETUP: gameStuff.sitrepFileName = >" + str(gameStuff.sitrepFileName) + "<\n")
        sys.stdout.write("MAPSETUP: gameStuff.counterPath = >" + str(gameStuff.counterPath) + "<\n")
        sys.stdout.write("MAPSETUP: gameStuff.mapPath = >" + str(gameStuff.mapPath) + "<\n")
        sys.stdout.write("MAPSETUP: gameStuff.ssdPath = >" + str(gameStuff.ssdPath) + "<\n")

    mapBackground = mapInfo.mBackFN
    mapFN = mapInfo.mOutFilename

    mapFile = open(mapFN,"w")

    mapFile.write('<html><head><title>'+ gameStuff.gameName)
    if ( not((gameStuff.currTurn == "0") or (gameStuff.currImpulse == "0"))):
        mapFile.write(' Turn:' + gameStuff.currTurn + ' Impulse:' + gameStuff.currImpulse)
    mapFile.write('</title>')
    mapFile.write('<META http-equiv=Content-Type content="text/html; charset=windows-1252">\n<STYLE>\n')
    for i in sideStuff:
        for j in ["left","right"]:
            mapFile.write('TD.' + i.desigLetter.lower() + 'Title' + j.title() + '{\n')
            mapFile.write('   FONT-SIZE: 12px; CURSOR: default; COLOR: ' + i.titleTextColor + ";")
            mapFile.write(' BACKGROUND-COLOR: '+ i.titleColor +'; TEXT-ALIGN: ' + j + '\n}\n')
            mapFile.write('TD.' + i.desigLetter.lower() + 'Body' + j.title() + '{\n')
            mapFile.write('   FONT-SIZE: 12px; CURSOR: default; COLOR: ' + i.bodyTextColor + ";")
            mapFile.write(' BACKGROUND-COLOR: '+ i.bodyColor +'; TEXT-ALIGN: ' + j + '\n}\n')

    mapFile.write('.ssdButton {')
    mapFile.write('	FONT-SIZE: 10px\n}\n')
    mapFile.write('TD.parentTitleLeft {\n')
    mapFile.write('	FONT-SIZE: 12px; CURSOR: default; COLOR: ' + gameStuff.titleTextColor + ";")
    mapFile.write(' BACKGROUND-COLOR: '+ gameStuff.titleColor +'; TEXT-ALIGN: center\n}\n')
    mapFile.write('TD.parentTitleRight {\n')
    mapFile.write('	FONT-SIZE: 12px; CURSOR: default; COLOR: ' + gameStuff.titleTextColor + ";")
    mapFile.write(' BACKGROUND-COLOR: '+ gameStuff.titleColor +'; TEXT-ALIGN: right\n}\n')
    mapFile.write('TD.parentBody {\n')
    mapFile.write('	FONT-SIZE: 12px; CURSOR: default; COLOR: ' + gameStuff.bodyTextColor + ";")
    mapFile.write(' BACKGROUND-COLOR: '+ gameStuff.bodyColor +'; TEXT-ALIGN: left;')
    mapFile.write(' TEXT-DECORATION: none\n}\n')
    mapFile.write('A.parentLink {\n')
    mapFile.write('	COLOR: ' + gameStuff.bodyTextColor + ";")
    mapFile.write(' BACKGROUND-COLOR: '+ gameStuff.bodyColor +'; TEXT-ALIGN: center;')
    mapFile.write(' TEXT-DECORATION: none\n}\n')

    mapFile.write('</STYLE>\n')
    mapFile.write('<SCRIPT LANGUAGE="JavaScript1.2" SRC="layers3.js"></SCRIPT></head>\n')
    mapFile.write('<body bgcolor="#000000" onload="JavaScript:initializeLayers();"><span style="position: absolute; left: 0; top: 0; z-index: 0"><img border="0" name="map" src="')
    mapFile.write(gameStuff.mapPath + mapBackground)
    mapFile.write('" width="' + mapInfo.mWidth + '" height="' + mapInfo.mHeight + '">')
    mapFile.write('</span>\n\n')

    hexList = gameStuff.positionList.keys()
    hexList.sort()

    for j in hexList:
        hexData = gameStuff.positionList[j]
        unitList = hexData.unitList
        if (ShowComments):
            sys.stdout.write("\nBUILDMAP : j = >" + str(j) + "< hexData.unitList = " + str(hexData.unitList) + "\n")
        
        mapStuff = buildMapHex(j,hexData,mapInfo,gameStuff)
        mapStuff += buildParentBox(j,hexData,mapInfo,gameStuff)
        if (len(unitList) > 1):
            for i in unitList:
                mapStuff += buildChildBox(i,hexData,mapInfo,gameStuff)
                if (ShowComments):
                    sys.stdout.write("\nBUILDMAP : i = " + str(i) + "\n")


        mapFile.write(mapStuff)

        if (ShowComments):
            sys.stdout.write("\nBUILDMAP : mapStuff = " + mapStuff + "\n")

    mapFile.write('</body></html>')
    mapFile.close()

# End of buildMapFile


def getUnitClass(unitIDSplit):

    ShowComments = False

    uClass = ""
    if (len(unitIDSplit) == 1):
        uID = unitIDSplit[0][1:]
        for i in uID:
            if (i.lower() in "abcdefghijklmnopqrstuvwxyz"):
                uClass += i
            else:
                pass #error condition
    else:
        for i in unitIDSplit[len(unitIDSplit) - 1]:
            if (i.lower() in "abcdefghijklmnopqrstuvwxyz"):
                uClass += i.lower()
            if (uClass == "h"):
                uClass = ""
                for i in unitIDSplit[len(unitIDSplit) - 2]:
                    if (i.lower() in "abcdefghijklmnopqrstuvwxyz"):
                        uClass += i.lower()
            
        uClass = "." +uClass

    if (ShowComments):
        sys.stdout.write("\nGETUNITCLASS : unitIDSplit = " + str(unitIDSplit) + "uClass = " + uClass)

    return uClass.lower()


# End of buildMapFile


def buildTipLines(fieldSplit,unitInfo):

    outLines = ""
    
    for i in fieldSplit:

        skipField = False
        currentField = i.strip().lower()
        
        if (currentField == "unit"):
            dData = unitInfo.unitID

        elif (currentField == "class"):
            dData = " Class:" + unitInfo.unitClass
                
        elif (currentField == "name"):
            dData = " Name:" + unitInfo.unitID
                
        elif (currentField == "hex"):
            dData = " Hex:" + unitInfo.hex
                
        elif (currentField == "speed"):
            dData = " Speed:" + unitInfo.speed
                
        elif (currentField == "ecm"):
            dData = " ECM:" + unitInfo.ecm
                
        elif (currentField == "eccm"):
            dData = " ECCM:" + unitInfo.eccm
                
        elif (currentField == "ew"):
            dData = " EW:" + unitInfo.ecm + "/" + unitInfo.eccm
                
        elif (currentField == "launch"):
            dData = " Launched:" + unitInfo.launch

        elif (currentField == "laid"):
            dData = " Laid:" + unitInfo.launch

        elif (currentField == "cast"):
            dData = " Cast:" + unitInfo.launch
                
        elif (currentField == "notes"):
            if (unitInfo.notes == ""):
                skipField = True
            else:
                dData = "\n  Notes:" + unitInfo.notes
                
        else:
            skipField = True

        if (not(skipField)):
            outLines += dData

    return outLines


# End of buildTipLines


def ShiftHex(startHex, shift, direction):


    ShowComments = False

    if(ShowComments):
        sys.stdout.write("\nSHIFTHEX: startHex = >" + startHex + "< shift = >" + str(shift) + "< direction = >"+ direction + "<\n")


    shift = int(shift)
    direction = direction.lower()

    hexCol= int(startHex[0:2])
    isOdd = hexCol%2
    IsEven = 1 - isOdd
    hexRow = int(startHex[2:4])

    if(ShowComments):
        sys.stdout.write("SHIFTHEX: hexCol = >" + str(hexCol) + "< hexRow = >" + str(hexRow) + "< isOdd = "+ str(isOdd) + "< IsEven = "+ str(IsEven) + "<\n")


    if (direction == "a"):
        hexRow -= shift

    if (direction == "d"):
        hexRow += shift

    if ((direction == "b") or (direction == "c")):
        hexCol += shift

    if ((direction == "e") or (direction == "f")):
        hexCol -= shift
 
    if ((direction == "b") or (direction == "f")):
        hexRow -= ((shift + isOdd) / 2)

    if ((direction == "c") or (direction == "e")):
        hexRow += ((shift + IsEven) / 2)


    newCol = str(hexCol)
    if (len(newCol) == 1):
        newCol = "0" + newCol

    newRow = str(hexRow)
    if (len(newRow) == 1):
        newRow = "0" + newRow

    newhex = newCol + newRow

    if(ShowComments):
        sys.stdout.write("SHIFTHEX: newCol = >" + newCol + "< hnewRow = >" + newRow + "< newhex = "+ newhex + "<\n\n")
    
        
    return newCol + newRow

# End of Shifthex


def AddUnit(unitType, unitStuff, currentSide, gameStuff):

    ShowComments = False

    unitID = unitStuff.unitID

    if (currentSide.unitList.has_key(unitID)):
        sys.stdout.write("ADDUNIT ERROR: UnitID already exists in side's unit list! unitID = >" + unitID + "<\n")
    else:
        currentSide.unitList[unitID] = unitStuff

    unitHexOnly = currentSide.unitList[unitID].hex
    if (not (gameStuff.positionList.has_key(unitHexOnly))):
        gameStuff.positionList[unitHexOnly] = HexStuff()

    if (gameStuff.classLookup.has_key(unitType)):
       displayData = gameStuff.classLookup[unitType]
       tipSplit = displayData.tooltip.split(",")
       currentSide.unitList[unitID].tooltipStuff = buildTipLines(tipSplit,unitStuff)
       if(ShowComments):
           sys.stdout.write("ADDUNIT: Found unit type unitType = >" + unitType + "< displayData.tooltip = >"  + displayData.tooltip + "<\n")
           sys.stdout.write("ADDUNIT: currentSide.unitList[unitID].tooltipStuff = >" + str(currentSide.unitList[unitID].tooltipStuff) + "<\n")

       
    elif(ShowComments):
        sys.stdout.write("ADDUNIT: Could not find unit type! unitType = >" + unitType + "<\n")
        

    hexStuff = gameStuff.positionList[unitHexOnly]
    if (hexStuff.unitList == []):
        hexStuff.unitList = [unitID]
        hexStuff.tooltips = currentSide.unitList[unitID].tooltipStuff
        
    else:
        hexStuff.unitList += [unitID]
        hexStuff.tooltips += '\n' + currentSide.unitList[unitID].tooltipStuff

    

# End of AddUnit

def readSitrep(gameStuff, sideStuff,mapStuff):

    ShowComments = False

    excelData = gameStuff.excelData
    nSides = gameStuff.nSides

    if(ShowComments):
        sys.stdout.write("READSITREP: nSides = " + str(nSides) + "< sitrep path and filename = >" + gameStuff.sitrepPath + gameStuff.sitrepFileName + "<\n\n")

    for i in sideStuff:
        currentSide = i
        sheetName = currentSide.sideName
        sheetBack = excelData.Set_Sheet_By_Name(sheetName)

        if(ShowComments):
            sys.stdout.write("\n\nREADSITREP: sheetName = >" + sheetName + "< sheetback = " + "< sheetBack = >" + str(sheetBack) + "<\n")
        
        if (sheetName != sheetBack):
            sys.stdout.write("READSITREP ERROR: could not find sheet >" + str(sheetName)+ "<\n")
        else:
            row = 0
            col = 0
            headingList = excelData.Get_Horiz_Strip(row)

            if(ShowComments):
                sys.stdout.write("READSITREP: headingList = >" + str(headingList) + "<\n")

            unitCol = headingList.index("Unit")
            unitList = excelData.Get_Vert_Strip(unitCol)

            if(ShowComments):
                sys.stdout.write("READSITREP: unitCol = >" + str(unitCol) + "< unitList = >" + str(unitList) + "< \n")

            row = 2
            curentImpulse = str(int(gameStuff.currImpulse))
            
            for j in range(row, len(unitList)):
                unitStrip = excelData.Get_Horiz_Strip(j)
                unitStuff = UnitStuff()
                if(len(unitStrip[0].strip()) > 2):
                    for k in range(0, len(unitStrip)):
                        colHeading = headingList[k].strip().lower()
         
                        if (ShowComments):
                            sys.stdout.write("READSITREP: k = >" + str(k) + "< colHeading = >" + str(colHeading) + "< unitStrip[k] = >" + str(unitStrip[k]) + "<\n")

                        if (colHeading == "unit"):
                            unitID = unitStrip[k].strip()
                            unitStuff.unitID = unitID
                            
                            if (ShowComments):
                                sys.stdout.write("READSITREP: unitID = >" + unitID + "<\n")

                            if ("*" in unitID) :
                                unitID = unitID.split("*")[0]
                            unitIDSplit = unitID.split(".")
                            unitType = getUnitClass(unitIDSplit)

                            unitStuff.isBackground = (unitType in gameStuff.backgroundClasses)

                            # what the excel sheet calls "Type", I put in "unitClass" (see below)
                            # unit Type is extracted from unitID ("Unit" in the excel sheet).
                            # this note here is to remind anyone working on thsi code that "Type" and "class:
                            # are reversed from their use in the spereadsheet.  I'm taking the easy way out
                            # and writing a reminder rather than actually resolving the issue.

                            if (ShowComments):
                                sys.stdout.write("READSITREP: unitType = >" + unitType + "<\n")
                                sys.stdout.write("READSITREP: unitStuff.unitID = >" + unitStuff.unitID + "<\n")

                        elif (colHeading == "type"):
                            unitStuff.unitClass = unitStrip[k]
                            
                        elif (colHeading == "speed"):
                            unitStuff.speed = str(int(unitStrip[k]))
                            
                        elif (colHeading == "ecm"):
                            unitStuff.ecm = str(int(unitStrip[k]))
                            
                        elif (colHeading == "eccm"):
                            unitStuff.eccm = str(int(unitStrip[k]))
                            
                        elif (colHeading == "publicnotes"):
                            unitStuff.notes = unitStrip[k]

                        elif (colHeading == "counterfilename"):
                            unitStuff.counterFn = unitStrip[k]
                            
                        elif (colHeading == "ssdfilename"):
                            unitStuff.ssdFn = unitStrip[k]
                            
                        elif (colHeading == ("imp" + curentImpulse)):

                            if (ShowComments):
                                sys.stdout.write("READSITREP: curentImpulse = >" + str(curentImpulse) + "< colHeading = >" + colHeading + "<\n")

                            currHex = unitStrip[k].strip()

                            if (ShowComments):
                                sys.stdout.write("READSITREP: currHex = >" + str(currHex) + "<\n")

                            if ("NM" in currHex):
                                l = k - 1
                                
                                if (ShowComments):
                                    sys.stdout.write("READSITREP: l = >" + str(l) + "<\n")
                                
                                while(("-" in unitStrip[l]) or ("NM" in unitStrip[l])):
                                    l -= 1
                                
                                    if (ShowComments):
                                        sys.stdout.write("READSITREP: l = >" + str(l) + "< unitStrip[l] = >" + unitStrip[l] + "<\n")
                                
                                currHex = unitStrip[l]
                                unitStuff.facing = currHex[4]
                                unitStuff.hex = currHex[0:4]

                                if (ShowComments):
                                    sys.stdout.write("READSITREP: currHex = >" + str(currHex) + "<\n")

                                   
                            elif (len(currHex) > 4):
                                unitStuff.facing = currHex[4]
                                unitStuff.hex = currHex[0:4]

                    if ((unitStuff.facing != "Z") and (unitStuff.hex != "0000")and (unitStuff.hex != "")):
                        AddUnit(unitType, unitStuff, currentSide, gameStuff)

                        unitNotes = unitStuff.notes.lower()
                        
                        if ("esg" in unitNotes):
                            noteSplit = unitNotes.split("esg")
                            del noteSplit[0]
                            for m in  noteSplit:
                                esgSplit = m.split(",")
                                for n in esgSplit:
                                    esgRadiusStr = n.strip()
                                    ersLen = len(esgRadiusStr)
                                    if (ersLen == 0):
                                        continue
                                    elif (ersLen > 1):
                                        nList = list(esgRadiusStr)
                                        for p in nList:
                                            if (p.isdigit()):
                                                esgRadiusStr = p
                                    if (esgRadiusStr.isdigit()):
                                        esgRadiusInt = int(esgRadiusStr)
                                        # determine if ESG radius is 1-3. 0 is not displayed at this time.
                                        if ((esgRadiusInt <4) and (esgRadiusInt > 0)):
                                            directions = ("a","b","c","d","e","f")

                                            # Do ESG corners
                                            esgCount = 0
                                            for p in range(0,6):
                                                esgCount = esgCount + 1
                                                esgStuff = UnitStuff()
                                                esgStuff.unitID = unitStuff.unitID + ".esg" + esgRadiusStr + ".e" + str(esgCount)
                                                esgStuff.unitClass = "ESG"
                                                esgStuff.speed = unitStuff.speed
                                                esgStuff.notes = unitStuff.unitID + " ESG radius " + esgRadiusStr
                                                esgStuff.useNeutral = True
                                                esgStuff.counterFn = "esg"
                                                esgStuff.oddCounterSize = True
                                                esgStuff.formatCode = "h:"
                                                esgStuff.facing = directions[p]
                                                StraightPieceMoveDir = directions[(p + 2)%6]
                                                esgHex= ShiftHex(unitStuff.hex, esgRadiusInt, esgStuff.facing)
                                                esgStuff.hex = esgHex
                                                AddUnit(".e", esgStuff, currentSide, gameStuff)

                                                # Fill in between corners for radii 2 and 3
                                                if (esgRadiusInt > 1):
                                                    for q in range(1, esgRadiusInt):
                                                        esgCount = esgCount + 1
                                                        esgStuff = UnitStuff()
                                                        esgStuff.unitID = unitStuff.unitID + ".esg" + esgRadiusStr + ".e" + str(esgCount)
                                                        esgStuff.unitClass = "ESG"
                                                        esgStuff.speed = unitStuff.speed
                                                        esgStuff.notes = unitStuff.unitID + " ESG radius " + esgRadiusStr
                                                        esgStuff.useNeutral = True
                                                        esgStuff.counterFn = "esgs"
                                                        esgStuff.oddCounterSize = True
                                                        esgStuff.formatCode = "h:"
                                                        esgStuff.facing = directions[p]
                                                        esgStuff.hex = ShiftHex(esgHex, q, StraightPieceMoveDir)
                                                        AddUnit(".e", esgStuff, currentSide, gameStuff)
                                                        
                                                    # lay down straight ESG pieces
                                                

                                            # ShiftHex(startHex, shift, direction)


                                            
                                        elif(esgRadiusInt != 0):
                                            sys.stdout.write("READSITREP ERROR: ESG range too large or small! m = >" + m + " n = >" + n + "<\n")
                                    else:
                                        sys.stdout.write("READSITREP ERROR: ESG range not a digit! m = >" + m + " n = >" + n + "<\n")
                                


                        ##if ("tractoring" in unitNotes):



                            

    if (ShowComments):
        for i in range(gameStuff.nSides+1) :
            sys.stdout.write(" sideStuff[" + str(i) + "].sideName = >"+ sideStuff[i].sideName+ "<\n")
            sys.stdout.write("readSitrep sideStuff[" + str(i) + "] unitStuff keys:" + str(sideStuff[i].unitList.keys()) + "\n")
            sys.stdout.write("\n\n")

        sys.stdout.write(" gameStuff.positionList.keys() = >"+ str(gameStuff.positionList.keys()) + "<\n")


# End of readSitrep


def AddExcelStuff(gameStuff, sideStuff, excelData, dataStuff, mapStuff):

    ShowComments = False

    sheetname = excelData.Set_Sheet_By_Name("Mapmaker")
    if (sheetname != "Mapmaker"):
        sys.stdout.write("AddExcelEntry ERROR Could not Find Mapmaker sheet! Xlbook return value is>")
        sys.stdout.write(sheetname)
        sys.stdout.write("<\n")

    else:
        value = excelData.Get_Cell_Value()

        if (ShowComments):
            sys.stdout.write("AddExcelEntry: Mapmaker sheet, cell at 0,0 value = >")
            sys.stdout.write(value)
            sys.stdout.write("<\n")

    #   Read Number of sides, map paths

        col = excelData.SetCol(0)
        row = excelData.SetRow(0)
        eNames = excelData.Get_Vert_Strip()
        eValues = excelData.Get_Vert_Strip(1)
        value = excelData.Get_Cell_Value()

        if (ShowComments):
            sys.stdout.write("AddExcelEntry: eNames = >" + str(eNames) + "<\n")
            sys.stdout.write("AddExcelEntry: eValues = >" + str(eValues) + "<\n")
        
        nLength = len(eNames)
        vLength = len(eValues)
        if (nLength  < vLength):
            eLength = nLength - 1
        else:
            eLength = vLength - 1

        nSides = -1
        col = excelData.SetCol(1)

        for row in range(0, eLength-1):

            if (ShowComments):
                sys.stdout.write("AddExcelEntry: row = >" + str(row) + "<\n")
                sys.stdout.write("AddExcelEntry: eNames[row] = >" + str(eNames[row]) + "<\n")
                sys.stdout.write("AddExcelEntry: eValues[row] = >" + str(eValues[row]) + "<\n")


            # Recording game name and current turn, impulse

            eName = eNames[row].strip()

            if (eName == "Game Name"):
                gameStuff.gameName = eValues[row].strip()

            if (eName == "Turn"):
                currTurn = str(eValues[row]).strip()
                if ("." in currTurn):
                    currSplit = currTurn.split(".")
                    currTurn = currSplit[0]
                if (len(currTurn) == 1):
                    currTurn = "0" + currTurn
                gameStuff.currTurn = currTurn

            if (eName == "Impulse"):
                currImpulse = str(eValues[row]).strip()
                if ("." in currImpulse):
                    currSplit = currImpulse.split(".")
                    currImpulse = currSplit[0]
                if (len(currImpulse) == 1):
                    currImpulse = "0" + currImpulse
                gameStuff.currImpulse = currImpulse

            # Recording Side and path info

            if (eName == "Number of Sides"):
                gameStuff.nSides = int(eValues[row])
                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: gameStuff.nSides = >" + str(gameStuff.nSides) + "<\n")
                
            if (eName == "Map Path"):
                gameStuff.mapPath = eValues[row].strip()
                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: gameStuff.mapPath = >" + str(gameStuff.mapPath) + "<\n")

            if (eName == "SSD path"):
                gameStuff.ssdPath = eValues[row].strip()
                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: gameStuff.ssdPath = >" + str(gameStuff.ssdPath) + "<\n")
                
            if (eName == "Counter Path"):
                gameStuff.counterPath = eValues[row].strip()
                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: gameStuff.counterPath = >" + str(gameStuff.counterPath) + "<\n")

            # Begin recording side information NOTE: Plays with current cell in excelData!

            if (eName == "Side Name"):
                currentSide = SideStuff()
                sideStuff.append(currentSide)
                nSides += 1

                currentSide.sideName = eValues[row].strip()

                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: NEW SIDE: nSides = >" + str(nSides) + "<\n")
                    sys.stdout.write("AddExcelEntry: currentSide.sideName = >" + str(currentSide.sideName) + "<\n")
                    sys.stdout.write("AddExcelEntry: eValues[row].strip() = >" + str(eValues[row].strip()) + "<\n")

                
            if (eName == "Counter subfolder"):
                currentSide.counterFolder = eValues[row].strip()

                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: currentSide.counterFolder = >" + str(currentSide.counterFolder) + "<\n")
                    sys.stdout.write("AddExcelEntry: eValues[row].strip() = >" + str(eValues[row].strip()) + "<\n")

            if ("Side Letter" in eName):
                currentSide.desigLetter = eValues[row].strip()
                gameStuff.validSides[currentSide.desigLetter] = currentSide

                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: currentSide.desigLetter = >" + str(currentSide.desigLetter) + "<\n")
                    sys.stdout.write("AddExcelEntry: eValues[row].strip() = >" + str(eValues[row].strip()) + "<\n")



            if (eName == "Hex Index Box"):
                check = excelData.SetRow(row)
                if (ShowComments):
                    cellValue = excelData.Get_Cell_Value()
                    sys.stdout.write("AddExcelEntry: cellValue = >" + str(cellValue) + "<\n")

                if (check == row):
                    gameStuff.titleTextColor = excelData.Cell_Font_Color(row)
                    gameStuff.titleColor = excelData.Cell_BG_Color(row)
                    check = excelData.SetRow(row+1)
                else:
                    sys.stdout.write("AddExcelEntry ERROR Could not find cell in excelBook for index box header colors")
                    sys.stdout.write("AddExcelEntry: nSides =  = >"+ str(nSides) +"< row = >" + str(row) +"< check = >"+ str(check) +"<\n")

                row += 1
                check = excelData.SetRow(row)
                if (ShowComments):
                    cellValue = excelData.Get_Cell_Value()
                    sys.stdout.write("AddExcelEntry: cellValue = >" + str(cellValue) + "<\n")
                
                if (check == row):
                    gameStuff.bodyTextColor = excelData.Cell_Font_Color(row)
                    gameStuff.bodyColor = excelData.Cell_BG_Color(row)
                else:
                    sys.stdout.write("AddExcelEntry ERROR Could not find cell in excelBook for index box body colors")
                    sys.stdout.write("AddExcelEntry: nSides =  = >"+ str(nSides) +"< row = >" + str(row) +"< check = >"+ str(check) +"<\n")


            if (eName == "Info Box Colors"):
                check = excelData.SetRow(row)
                if (ShowComments):
                    cellValue = excelData.Get_Cell_Value()
                    sys.stdout.write("AddExcelEntry: cellValue = >" + str(cellValue) + "<\n")
                
                if (check == row):
                    currentSide.titleTextColor = excelData.Cell_Font_Color(row)
                    currentSide.titleColor = excelData.Cell_BG_Color(row)
                    check = excelData.SetRow(row+1)
                else:
                    sys.stdout.write("AddExcelEntry ERROR Could not find cell in excelBook for info box header colors")
                    sys.stdout.write("AddExcelEntry: nSides =  = >"+ str(nSides) +"< row = >" + str(row) +"< check = >"+ str(check) +"<\n")

                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: currentSide.titleTextColor = >" + str(currentSide.titleTextColor) + "< ")
                    sys.stdout.write("AddExcelEntry: currentSide.titleColor = >" + str(currentSide.titleColor) + "<\n")

                row += 1
                check = excelData.SetRow(row)
                if (ShowComments):
                    cellValue = excelData.Get_Cell_Value()
                    sys.stdout.write("AddExcelEntry: cellValue = >" + str(cellValue) + "<\n")
                
                if (check == row):
                    currentSide.bodyTextColor = excelData.Cell_Font_Color(row)
                    currentSide.bodyColor = excelData.Cell_BG_Color(row)
                else:
                    sys.stdout.write("AddExcelEntry ERROR Could not find cell in excelBook for info box body colors")
                    sys.stdout.write("AddExcelEntry: nSides =  = >"+ str(nSides) +"< row = >" + str(row) +"< check = >"+ str(check) +"<\n")

                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: currentSide.bodyTextColor = >" + str(currentSide.titleTextColor) + "< ")
                    sys.stdout.write("AddExcelEntry: currentSide.bodyColor = >" + str(currentSide.titleColor) + "<\n")


# Begin adding Info box and tooltip diaply fields.  Fields must correspond to
# a column heading in the excel file.

        row = 0
        col = excelData.Find_Col_By_Value("DATA", row)
        if (col < 0):
            sys.stdout.write("AddExcelEntry: Could not find DATA column for Info Box/tooltip fields col = " + str(col) +"<\n")
        else:
            dataColumn = excelData.Get_Vert_Strip(col)
            typesColumn = excelData.Get_Vert_Strip(col+1)
            infoBoxColumn = excelData.Get_Vert_Strip(col+2)
            tooltipColumn = excelData.Get_Vert_Strip(col+3)
            mapFileColumn = excelData.Get_Vert_Strip(col+4)

            if (ShowComments):
                sys.stdout.write("AddExcelEntry: dataColumn = >" + str(dataColumn) + "<\n")
                sys.stdout.write("AddExcelEntry: typesColumn = >" + str(typesColumn) + "<\n")
                sys.stdout.write("AddExcelEntry: infoBoxColumn = >" + str(infoBoxColumn) + "<\n")
                sys.stdout.write("AddExcelEntry: tooltipColumn = >" + str(tooltipColumn) + "<\n")
            
            stopRow = excelData.Find_Row_By_Value("MAP INFORMATION",col)-1
            if (stopRow < 0):
                sys.stdout.write("AddExcelEntry: Could not find end point for nfo Box/tooltip fields  stopRow = " + str(stopRow) +"<\n")
                sys.stdout.write("AddExcelEntry: dataColumn = " + str(dataColumn) +"<\n")
                
            else:
                if (ShowComments):
                    sys.stdout.write("AddExcelEntry: stopRow = >" + str(stopRow) + "<\n")

                for i in range(1,stopRow):
                    if (dataColumn[i] != ""):
                        if (ShowComments):
                            sys.stdout.write("AddExcelEntry: i = >" + str(i) +  "< dataColumn[i] = >" + str(dataColumn[i]) +"<\n")
                        
                        unitCategory = ""
                        unitTypes = ""
                        infoBox = ""
                        tooltips = ""

                        unitCategory = (dataColumn[i].strip()).lower()
                        unitTypes = typesColumn[i].strip()
                        
                        if (ShowComments):
                            sys.stdout.write("AddExcelEntry: unitCategory = >" + str(unitCategory) +  "< unitTypes = >" + str(unitTypes) +"<\n")

                        j = i
                        while (infoBoxColumn[j] != ""):
                            if (ShowComments):
                                sys.stdout.write("AddExcelEntry: j = >" + str(j) +  "< infoBoxColumn[j] = >" + str(infoBoxColumn[j]) +"<\n")
                            
                            infoBox += (infoBoxColumn[j].strip() + ",")

                            j += 1

                        # trailing "," not removed.  If this is a problem, remove it here.

                        j = i
                        while (tooltipColumn[j] != ""):
                            if (ShowComments):
                                sys.stdout.write("AddExcelEntry: j = >" + str(j) +  "< tooltipColumn[j] = >" + str(tooltipColumn[j]) +"<\n")
                            tooltips += (tooltipColumn[j].strip() + ",")
                            j += 1

                        # trailing "," not removed.  If this is a problem, remove it here.

                        df = DataFields()
                        df.boxTitle = "Unit"
                        df.boxData = infoBox
                        df.tooltip = tooltips

                        typeSplit = unitTypes.split(",")

                        for i in typeSplit:
                            uType = i.strip().lower()
                            dataStuff.append(df)
                            gameStuff.classLookup[uType] = df
                            if (unitCategory != "moving"):
                                gameStuff.backgroundClasses.append(uType)
                            
                            if (ShowComments):
                                sys.stdout.write("AddExcelEntry: Adding Data Field utype = >" + str(uType) + "<\n")
                                sys.stdout.write("AddExcelEntry: gameStuff.classLookup.keys() = >" + str(gameStuff.classLookup.keys()) + "<\n")


                placeColumn = dataColumn
                hexMapColumn = typesColumn
                # oddly enough infoBoxColumn and tooltipcolumn are the same for
                # for map info as they were for display data above.  mapFileColumn
                # is defined above
                
                for i in range(stopRow, (len(dataColumn)-1)):
                    placeScheme = str(placeColumn[i])
                    if ("." in placeScheme):
                        psSplit = placeScheme.split(".")
                        placeScheme = psSplit[0]
                    if (placeScheme.isdigit()):
                        newMap = MapStuff(placeScheme)
                   # change below for newMap.properties
                        newMap.mType = placeScheme
                        newMap.mBackFN = hexMapColumn[i].strip()
                        newMap.mInfoBoxes = (infoBoxColumn[i].strip() == "yes")
                        newMap.mToolTips = (tooltipColumn[i].strip() == "yes")
                        newMap.MapfileName(mapFileColumn[i], gameStuff)
                        mapStuff.append(newMap)
                        



# End of AddExcelStuff



          
def mapSetup():

    ShowComments = False
    
    gameStuff = GameStuff()
    sideStuff = []
    mapStuff = []
    dataStuff = []
    errors = ""

    iniStuff = iniRead("","mapmaker.ini")

    for iniEntry in iniStuff:

        processThis = False
        errorMessage = False
        varName = iniEntry[0].strip()
        varInfo = iniEntry[1].strip()

        if (ShowComments):
            sys.stdout.write("iniEntry: >")
            sys.stdout.write(iniEntry)
            sys.stdout.write("< varName: >")
            sys.stdout.write(varName)
            sys.stdout.write("< varInfo: >")
            sys.stdout.write(varInfo)
            sys.stdout.write("<\n")

        if (varName == "game.sitrepFilename"):
            splitstuff = '"'
            varSplit = varInfo.split(splitstuff)
            gameStuff.sitrepFileName = varSplit[1]


    if (ShowComments):
        sys.stdout.write("MAPSETUP: gameStuff.sitrepFileName = >" + str(gameStuff.sitrepFileName) + "<\n")

    excelData = xlbook.XLBOOK(gameStuff.sitrepFileName)
    gameStuff.excelData = excelData

    AddExcelStuff(gameStuff, sideStuff, excelData, dataStuff, mapStuff)

    if (ShowComments):
        sys.stdout.write("MAPSETUP: gameStuff.sitrepFileName = >" + str(gameStuff.sitrepFileName) + "<\n")
        sys.stdout.write("MAPSETUP: gameStuff.counterPath = >" + str(gameStuff.counterPath) + "<\n")
    
    readSitrep(gameStuff, sideStuff, mapStuff)

    if (ShowComments):
        for i in range(gameStuff.nSides+1) :
            sys.stdout.write("MAPSETUP sideStuff[")
            sys.stdout.write(i)
            sys.stdout.write("] keys:")
            sys.stdout.write(sideStuff[i].unitList.keys())
            sys.stdout.write("\n\n")
            
        sys.stdout.write(" gameStuff.positionList.keys() = >"+ str(gameStuff.positionList.keys()) + "<\n\n")

        sys.stdout.write(" len(mapStuff) = >"+ str(len(mapStuff)) + "<\n")

    for i in mapStuff :
        buildMapFile(i, gameStuff, sideStuff, mapStuff, dataStuff)

    if (errors):
        sys.stdout.write("<\n")
        sys.stdout.write("Errors: ")
        sys.stdout.write(errors)
        sys.stdout.write("<\n")

# End of mapSetup
            
mapSetup()
